import b12od

def test_offer_decode():
  b12j = b12od.Bolt12OfferDecoder()
  expected = "{\"offer_description\":\"Test vectors\",\"offer_paths\":[{\"first_node_id\":\"0324653eac434488002cc06bbfb7f10fe18991e35f9fe4302dbea6d2353dc0ab1c\",\"first_path_key\":\"020202020202020202020202020202020202020202020202020202020202020202\",\"path\":[{\"blinded_node_id\":\"020202020202020202020202020202020202020202020202020202020202020202\",\"encrypted_recipient_data\":\"00000000000000000000000000000000\"},{\"blinded_node_id\":\"020202020202020202020202020202020202020202020202020202020202020202\",\"encrypted_recipient_data\":\"1111111111111111\"}]},{\"first_scid\":\"1x2x3\",\"first_scid_dir\":1,\"first_path_key\":\"020202020202020202020202020202020202020202020202020202020202020202\",\"path\":[{\"blinded_node_id\":\"020202020202020202020202020202020202020202020202020202020202020202\",\"encrypted_recipient_data\":\"00000000000000000000000000000000\"},{\"blinded_node_id\":\"020202020202020202020202020202020202020202020202020202020202020202\",\"encrypted_recipient_data\":\"2222222222222222\"}]}],\"offer_issuer_id\":\"02eec7245d6b7d2ccb30380bfbe2a3648cd7a942653f5aa340edcea1f283686619\"}"
  assert b12j.decode("lno1pgx9getnwss8vetrw3hhyucsl5qj5qeyv5l2cs6y3qqzesrth7mlzrlp3xg7xhulusczm04x6g6nms9trspqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqqsqqqqqqqqqqqqqqqqqqqqqqqqqqpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqsqpqg3zyg3zyg3zygpqqqqzqqqqgqqxqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqqgqqqqqqqqqqqqqqqqqqqqqqqqqqqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgpqyqszqgqqsg3zyg3zyg3zygtzzqhwcuj966ma9n9nqwqtl032xeyv6755yeflt235pmww58egx6rxry") == expected, "Should be " + expected



cdef extern from "b12od/bolt12_offer_json.h":
    cdef struct bolt12_json
    bolt12_json* bolt12_offer_json_new()
    const char* bolt12_offer_json(bolt12_json* handle, const char* offer_string)
    void bolt12_offer_json_delete(bolt12_json* handle)

cdef class Bolt12OfferDecoder:
    cdef bolt12_json* b12j

    def __cinit__(self):
        self.b12j = bolt12_offer_json_new()
        if self.b12j == NULL:
            raise MemoryError("Failed to create handle")

    def __dealloc__(self):
        if self.b12j != NULL:
            bolt12_offer_json_delete(self.b12j)
            self.b12j = NULL

    def decode(self, offer_string: str) -> str:
        boffer_string = offer_string.encode("utf-8")
        cdef const char* coffer_string = boffer_string
        return (<bytes>bolt12_offer_json(self.b12j, coffer_string)).decode("utf-8")




import os
import subprocess
from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
from Cython.Build import cythonize

class CMakeBuildExt(build_ext):
    def run(self):
        # Run CMake to build the C library
        build_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "b12od-core/build"))
        os.makedirs(build_dir, exist_ok=True)
        source_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "b12od-core"))
        build_type = os.environ.get("BUILD_TYPE", "")

        if build_type == "":
          subprocess.check_call(["cmake", source_dir], cwd=build_dir)
        else:
          subprocess.check_call(["cmake", "-DCMAKE_BUILD_TYPE="+build_type, source_dir], cwd=build_dir)
        subprocess.check_call(["cmake", "--build", ".", "--target", "libb12od_pic"], cwd=build_dir)
        super().run()

    def build_extensions(self):
        lib_path = os.path.join(os.path.dirname(__file__), "b12od-core/build/libb12od_pic.a")

        for ext in self.extensions:
          ext.extra_objects = [lib_path]   # link the static archive directly
          ext.include_dirs.append("b12od-core")
        super().build_extensions()

c_flags = os.environ.get("CMAKE_C_FLAGS", "").split()

extensions = cythonize([
  Extension(
    "b12od.b12od",
    sources=["b12od/b12od.pyx"],
    include_dirs=["b12od-core/include"],   # so Cython sees bolt12_offer_decode.h
    language="c",
    extra_compile_args=c_flags
  )
])

setup(
    ext_modules=extensions,
    cmdclass={"build_ext": CMakeBuildExt},
    packages=["b12od"],
    zip_safe=False,
    install_requires=["pytest"],
)
